Package: ddpcr
Title: Analysis and Visualization of Droplet Digital PCR in R and on
        the Web
Version: 1.15.2
Authors@R: c(
    person("Dean", "Attali", email = "daattali@gmail.com", role = c("aut", "cre"),
    comment= c(ORCID="0000-0002-5645-3493"))
    )
Description: An interface to explore, analyze, and visualize droplet digital PCR
    (ddPCR) data in R. This is the first non-proprietary software for analyzing
    two-channel ddPCR data. An interactive tool was also created and is available
    online to facilitate this analysis for anyone who is not comfortable with
    using R.
URL: https://github.com/daattali/ddpcr,
        https://daattali.com/shiny/ddpcr/
BugReports: https://github.com/daattali/ddpcr/issues
Depends: R (>= 3.1.0)
Imports: DT (>= 0.2), dplyr (>= 0.5.0), ggplot2 (>= 2.2.0), lazyeval
        (>= 0.1.10), magrittr (>= 1.5), mixtools (>= 1.0.2), plyr (>=
        1.8.1), readr (>= 0.1.0), shiny (>= 0.11.0), shinydisconnect,
        shinyjs (>= 0.4.0), tibble
Suggests: ggExtra (>= 0.3.0), graphics, grid (>= 3.2.2), gridExtra (>=
        2.0.0), knitr (>= 1.7), rmarkdown, stats, testthat (>= 0.11.0),
        utils
License: MIT + file LICENSE
VignetteBuilder: knitr
RoxygenNote: 7.2.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2023-08-19 23:00:34 UTC; Dean-X1C
Author: Dean Attali [aut, cre] (<https://orcid.org/0000-0002-5645-3493>)
Maintainer: Dean Attali <daattali@gmail.com>
Repository: CRAN
Date/Publication: 2023-08-20 22:32:32 UTC
Built: R 4.2.0; ; 2023-08-27 04:14:04 UTC; unix
