# Generated by roxygen2: do not edit by hand

S3method(classify_droplets,pnpp_experiment)
S3method(classify_droplets_single,pnpp_experiment)
S3method(define_clusters,custom_thresholds)
S3method(define_clusters,ddpcr_plate)
S3method(define_clusters,pnpp_experiment)
S3method(define_params,custom_thresholds)
S3method(define_params,ddpcr_plate)
S3method(define_params,fam_positive_pnpp)
S3method(define_params,hex_positive_pnpp)
S3method(define_params,pnpp_experiment)
S3method(define_params,wildtype_mutant_pnpp)
S3method(define_steps,custom_thresholds)
S3method(define_steps,ddpcr_plate)
S3method(define_steps,pnpp_experiment)
S3method(diff,point2d)
S3method(format,point2d)
S3method(get_empty_cutoff,ddpcr_plate)
S3method(get_empty_cutoff,pnpp_experiment)
S3method(get_outlier_cutoff,ddpcr_plate)
S3method(is_well_success,ddpcr_plate)
S3method(is_well_success,pnpp_experiment)
S3method(parent_plate_type,ddpcr_plate)
S3method(parent_plate_type,default)
S3method(parent_plate_type,fam_positive_pnpp)
S3method(parent_plate_type,hex_positive_pnpp)
S3method(parent_plate_type,wildtype_mutant_pnpp)
S3method(plot,custom_thresholds)
S3method(plot,ddpcr_plate)
S3method(plot,pnpp_experiment)
S3method(plot,wildtype_mutant_pnpp)
S3method(print,ddpcr_plate)
S3method(print,point2d)
S3method(reclassify_droplets,pnpp_experiment)
S3method(reclassify_droplets_single,pnpp_experiment)
S3method(remove_empty,ddpcr_plate)
S3method(remove_empty,pnpp_experiment)
S3method(remove_failures,ddpcr_plate)
S3method(remove_outliers,ddpcr_plate)
S3method(subset,ddpcr_plate)
export("%>%")
export("%btwn%")
export("name<-")
export("negative_name<-")
export("params<-")
export("plate_data<-")
export("plate_meta<-")
export("positive_dim<-")
export("positive_name<-")
export("status<-")
export("thresholds<-")
export("variable_dim<-")
export("x_threshold<-")
export("x_var<-")
export("y_threshold<-")
export("y_var<-")
export(WELL_ID_REGEX)
export(analysis_complete)
export(analyze)
export(bind_df_ends)
export(calc_negative_freq_simple)
export(calculate_concentration)
export(calculate_concentration_single)
export(calculate_neg_freq_single)
export(calculate_negative_freqs)
export(capitalize)
export(cat0)
export(check_step)
export(classify_droplets)
export(classify_droplets_single)
export(classify_thresholds)
export(cluster)
export(cluster_name)
export(clusters)
export(col_to_num)
export(define_clusters)
export(define_params)
export(define_steps)
export(err_msg)
export(get_col)
export(get_empty_cutoff)
export(get_filled_border)
export(get_filled_drops)
export(get_outlier_cutoff)
export(get_row)
export(get_single_well)
export(get_wells_btwn)
export(has_signif_negative_cluster)
export(has_step)
export(is_dir)
export(is_dirty)
export(is_empty_plate)
export(is_file)
export(is_range)
export(is_well_success)
export(launch)
export(load_plate)
export(local_maxima)
export(local_minima)
export(lol_to_df)
export(mark_clusters)
export(merge_dfs_overwrite_col)
export(meta_var_name)
export(move_back)
export(move_front)
export(msg)
export(name)
export(named_vec_to_df)
export(negative_name)
export(new_plate)
export(next_step)
export(normalize_to_rds)
export(num_to_col)
export(num_to_row)
export(other_dim)
export(params)
export(parent_plate_type)
export(plate_data)
export(plate_meta)
export(plate_types)
export(point2d)
export(positive_dim)
export(positive_dim_var)
export(positive_name)
export(quiet)
export(range_list_to_vec)
export(range_to_endpoints)
export(range_to_seq)
export(reclassify_droplets)
export(reclassify_droplets_single)
export(remove_empty)
export(remove_failures)
export(remove_outliers)
export(reset)
export(row_to_num)
export(sample_data_dir)
export(sample_data_file)
export(sample_plate)
export(sample_results_file)
export(save_plate)
export(set_default_params)
export(set_thresholds)
export(status)
export(step)
export(step_begin)
export(step_end)
export(step_name)
export(steps)
export(thresholds)
export(type)
export(unanalyzed_clusters)
export(variable_dim)
export(variable_dim_var)
export(warn_msg)
export(well_info)
export(wells_failed)
export(wells_mutant)
export(wells_negative)
export(wells_positive)
export(wells_success)
export(wells_used)
export(wells_wildtype)
export(x_threshold)
export(x_var)
export(y_threshold)
export(y_var)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
